@@ -324,6 +324,50 @@ class VersionedObject(object):
         except AttributeError:
             return False
 
+    @classmethod
+    def to_json_schema(cls):
+        obj_name = cls.obj_name()
+        field_schemas = {key: field.get_schema()
+                         for key, field in cls.fields.items()}
+        required_fields = [name for name, schema in
+                           sorted(field_schemas.items())]
+        namespace_key = cls._obj_primitive_key('namespace')
+        name_key = cls._obj_primitive_key('name')
+        version_key = cls._obj_primitive_key('version')
+        data_key = cls._obj_primitive_key('data')
+        changes_key = cls._obj_primitive_key('changes')
+
+        schema = {
+            '$schema': 'http://json-schema.org/draft-04/schema#',
+            'title': obj_name,
+            'type': 'object',
+            'properties': {
+                namespace_key: {
+                    'type': 'string'
+                },
+                name_key: {
+                    'type': 'string'
+                },
+                version_key: {
+                    'type': 'string'
+                },
+                changes_key: {
+                    'type': 'array',
+                    'items': {
+                        'type': 'string'
+                    }
+                },
+                data_key: {
+                    'type': 'object',
+                    'description': 'fields of %s' % (obj_name),
+                    'properties': field_schemas,
+                    'required': required_fields
+                },
+            },
+            'required': [namespace_key, name_key, version_key, data_key]
+        }
+        return schema
+
     @classmethod
     def obj_name(cls):
         """Return the object's name
